@@ -1,5 +1,5 @@
 // Netify Agent
-// Copyright (C) 2015-2019 eGloo Incorporated <http://www.egloo.ca>
+// Copyright (C) 2015-2020 eGloo Incorporated <http://www.egloo.ca>
 //
 // This program is free software: you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@
 #define ND_FLOW_DHCPFP_LEN  48      // DHCP fingerprint length
 #define ND_FLOW_DHCPCI_LEN  96      // DHCP class identifier
 #define ND_FLOW_MDNS_ANSLEN 96      // MDNS answer length
+
 // BitTorrent info hash length
 #define ND_FLOW_BTIHASH_LEN SHA1_DIGEST_LENGTH
 
@@ -71,6 +72,8 @@ protected:
 typedef pair<const struct pcap_pkthdr *, const uint8_t *> nd_flow_push;
 typedef vector<nd_flow_push> nd_flow_capture;
 
+typedef unordered_map<string, string> nd_flow_kvmap;
+
 class ndFlow
 {
 public:
@@ -170,6 +173,10 @@ public:
         } mdns;
     };
 
+    struct {
+        nd_flow_kvmap headers;
+    } ssdp;
+
     enum {
         TYPE_LOWER,
         TYPE_UPPER,
@@ -227,6 +234,7 @@ public:
     bool has_ssl_server_ja3(void);
     bool has_bt_info_hash(void);
     bool has_mdns_answer(void);
+    bool has_ssdp_headers(void);
 
     void print(const char *tag, struct ndpi_detection_module_struct *ndpi);
 
