@@ -667,6 +667,10 @@ __attribute__ ((__packed__))
 
         /* NDPI_PROTOCOL_MEMCACHED */
         u_int8_t memcached_matches;
+
+        /* NDPI_PROTOCOL_WIREGUARD */
+        u_int8_t wireguard_stage;
+        u_int32_t wireguard_peer_index[2];
     }
 #ifndef WIN32
 __attribute__ ((__packed__))
