@@ -969,7 +969,7 @@ void getSSLorganization(struct ndpi_detection_module_struct *ndpi_struct,
 
             char *server_org = (char*)&packet->payload[i+5];
 
-            u_int len = (u_int)ndpi_min(server_len, buffer_len-1);
+            u_int len = (u_int)ndpi_min(server_len, sizeof(buffer)-1);
             strncpy(buffer, server_org, len);
             buffer[len] = '\0';
 
