@@ -182,12 +182,12 @@ size_t ndPacketQueue::push(struct pcap_pkthdr *pkt_header, const uint8_t *pkt_da
     if (ph == NULL) throw ndDetectionThreadException(strerror(ENOMEM));
     memcpy(ph, pkt_header, sizeof(struct pcap_pkthdr));
 
-    uint8_t *pd = new uint8_t[pkt_header->len];
+    uint8_t *pd = new uint8_t[pkt_header->caplen];
     if (pd == NULL) throw ndDetectionThreadException(strerror(ENOMEM));
-    memcpy(pd, pkt_data, pkt_header->len);
+    memcpy(pd, pkt_data, pkt_header->caplen);
 
     pkt_queue.push(make_pair(ph, pd));
-    pkt_queue_size += (sizeof(struct pcap_pkthdr) + pkt_header->len);
+    pkt_queue_size += (sizeof(struct pcap_pkthdr) + pkt_header->caplen);
 
 #ifdef _ND_LOG_PACKET_QUEUE
     nd_debug_printf("%s: packet queue push, new size: %lu\n",
@@ -229,7 +229,7 @@ void ndPacketQueue::pop(const string &oper)
     struct pcap_pkthdr *ph = pkt_queue.front().first;
     const uint8_t *pd = pkt_queue.front().second;
 
-    pkt_queue_size -= (sizeof(struct pcap_pkthdr) + ph->len);
+    pkt_queue_size -= (sizeof(struct pcap_pkthdr) + ph->caplen);
 
     delete ph;
     delete [] pd;
@@ -603,7 +603,7 @@ void ndDetectionThread::ProcessPacket(void)
     const uint8_t *l3 = NULL, *l4 = NULL, *pkt = NULL;
     uint16_t l2_len, l3_len, l4_len = 0, pkt_len = 0;
 
-    uint16_t type;
+    uint16_t type = 0;
     uint16_t ppp_proto;
     uint16_t frag_off = 0;
     uint8_t vlan_packet = 0;
@@ -636,12 +636,6 @@ void ndDetectionThread::ProcessPacket(void)
     stats->pkt.raw++;
     if (pkt_header->len > stats->pkt.maxlen)
         stats->pkt.maxlen = pkt_header->len;
-#if 0
-    if (pkt_header->caplen < pkt_header->len) {
-        // XXX: Warning: capture size less than packet size.
-        // XXX: Increase capture size (detection may not work)...
-    }
-#endif
 
     switch (pcap_datalink_type) {
     case DLT_NULL:
@@ -665,7 +659,7 @@ void ndDetectionThread::ProcessPacket(void)
             (hdr_eth->ether_dhost[0] == 0x01 && hdr_eth->ether_dhost[1] == 0x80 &&
             hdr_eth->ether_dhost[2] == 0xC2)) {
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->len;
+            stats->pkt.discard_bytes += pkt_header->caplen;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_debug_printf("%s: discard: STP protocol.\n", tag.c_str());
 #endif
@@ -680,9 +674,14 @@ void ndDetectionThread::ProcessPacket(void)
         l2_len = SLL_HDR_LEN;
         break;
 
+    case DLT_RAW:
+        l2_len = 0;
+        // type will be set to ETHERTYPE_IP/V6 below...
+        break;
+
     default:
         stats->pkt.discard++;
-        stats->pkt.discard_bytes += pkt_header->len;
+        stats->pkt.discard_bytes += pkt_header->caplen;
 #ifdef _ND_LOG_PKT_DISCARD
         nd_debug_printf("%s: discard: Unsupported datalink type: 0x%x\n",
             tag.c_str(), (unsigned)pcap_datalink_type);
@@ -722,7 +721,7 @@ void ndDetectionThread::ProcessPacket(void)
             );
             if (ppp_proto != PPP_IP && ppp_proto != PPP_IPV6) {
                 stats->pkt.discard++;
-                stats->pkt.discard_bytes += pkt_header->len;
+                stats->pkt.discard_bytes += pkt_header->caplen;
 #ifdef _ND_LOG_PKT_DISCARD
                 nd_debug_printf("%s: discard: unsupported PPP protocol: 0x%04hx\n",
                     tag.c_str(), ppp_proto);
@@ -735,7 +734,7 @@ void ndDetectionThread::ProcessPacket(void)
         else if (type == ETHERTYPE_PPPOEDISC) {
             stats->pkt.pppoe++;
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->len;
+            stats->pkt.discard_bytes += pkt_header->caplen;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_debug_printf("%s: discard: PPPoE discovery protocol.\n", tag.c_str());
 #endif
@@ -751,6 +750,9 @@ void ndDetectionThread::ProcessPacket(void)
     flow.ip_version = hdr_ip->ip_v;
 
     if (flow.ip_version == 4) {
+
+        if (type == 0) type = ETHERTYPE_IP;
+
         l3_len = ((uint16_t)hdr_ip->ip_hl * 4);
         l4_len = ntohs(hdr_ip->ip_len) - l3_len;
         flow.ip_protocol = hdr_ip->ip_p;
@@ -764,7 +766,7 @@ void ndDetectionThread::ProcessPacket(void)
         if (pkt_header->len - l2_len < sizeof(struct ip)) {
             // XXX: header too small
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->len;
+            stats->pkt.discard_bytes += pkt_header->caplen;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_debug_printf("%s: discard: header too small\n", tag.c_str());
 #endif
@@ -775,7 +777,7 @@ void ndDetectionThread::ProcessPacket(void)
             // XXX: fragmented packets are not supported
             stats->pkt.frags++;
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->len;
+            stats->pkt.discard_bytes += pkt_header->caplen;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_debug_printf("%s: discard: fragmented 0x3FFF\n", tag.c_str());
 #endif
@@ -786,7 +788,7 @@ void ndDetectionThread::ProcessPacket(void)
             // XXX: fragmented packets are not supported
             stats->pkt.frags++;
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->len;
+            stats->pkt.discard_bytes += pkt_header->caplen;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_debug_printf("%s: discard: fragmented 0x1FFF\n", tag.c_str());
 #endif
@@ -795,7 +797,7 @@ void ndDetectionThread::ProcessPacket(void)
 
         if (l3_len > (pkt_header->len - l2_len)) {
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->len;
+            stats->pkt.discard_bytes += pkt_header->caplen;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_debug_printf("%s: discard: l3_len[%hu] > (pkt_header->len[%hu] - l2_len[%hu])(%hu)\n",
                 tag.c_str(), l3_len, pkt_header->len, l2_len, pkt_header->len - l2_len);
@@ -805,7 +807,7 @@ void ndDetectionThread::ProcessPacket(void)
 
         if ((pkt_header->len - l2_len) < ntohs(hdr_ip->ip_len)) {
             stats->pkt.discard++;
-            stats->pkt.discard_bytes += pkt_header->len;
+            stats->pkt.discard_bytes += pkt_header->caplen;
 #ifdef _ND_LOG_PKT_DISCARD
             nd_debug_printf("%s: discard: (pkt_header->len[%hu] - l2_len[%hu](%hu)) < hdr_ip->ip_len[%hu]\n",
                 tag.c_str(), pkt_header->len, l2_len, pkt_header->len - l2_len, ntohs(hdr_ip->ip_len));
@@ -831,23 +833,32 @@ void ndDetectionThread::ProcessPacket(void)
                 memcpy(flow.upper_mac, hdr_eth->ether_shost, ETH_ALEN);
             }
         }
+
+        l4 = reinterpret_cast<const uint8_t *>(l3 + l3_len);
     }
     else if (flow.ip_version == 6) {
+
+        if (type == 0) type = ETHERTYPE_IPV6;
+
         hdr_ip6 = reinterpret_cast<const struct ip6_hdr *>(&pkt_data[l2_len]);
+
+        l3 = reinterpret_cast<const uint8_t *>(hdr_ip6);
         l3_len = sizeof(struct ip6_hdr);
+        l4 = reinterpret_cast<const uint8_t *>(l3 + l3_len);
         l4_len = ntohs(hdr_ip6->ip6_ctlun.ip6_un1.ip6_un1_plen);
         flow.ip_protocol = hdr_ip6->ip6_ctlun.ip6_un1.ip6_un1_nxt;
+
+        if (ndpi_handle_ipv6_extension_headers(NULL, &l4, &l4_len, &flow.ip_protocol)) {
+            stats->pkt.discard++;
+            stats->pkt.discard_bytes += pkt_header->caplen;
+#ifdef _ND_LOG_PKT_DISCARD
+            nd_debug_printf("%s: discard: Error walking IPv6 extensions.\n", tag.c_str());
+#endif
+            return;
+        }
+
         flow.lower_addr.ss_family = AF_INET6;
         flow.upper_addr.ss_family = AF_INET6;
-        l3 = reinterpret_cast<const uint8_t *>(hdr_ip6);
-
-        if (flow.ip_protocol == IPPROTO_DSTOPTS) {
-            const uint8_t *options = reinterpret_cast<const uint8_t *>(
-                hdr_ip6 + sizeof(const struct ip6_hdr)
-            );
-            flow.ip_protocol = options[0];
-            l3_len += 8 * (options[1] + 1);
-        }
 
         int i = 0;
         if (memcmp(&hdr_ip6->ip6_src, &hdr_ip6->ip6_dst, sizeof(struct in6_addr))) {
@@ -880,7 +891,7 @@ void ndDetectionThread::ProcessPacket(void)
     else {
         // XXX: Warning: unsupported IP protocol version (IPv4/6 only)
         stats->pkt.discard++;
-        stats->pkt.discard_bytes += pkt_header->len;
+        stats->pkt.discard_bytes += pkt_header->caplen;
 #ifdef _ND_LOG_PKT_DISCARD
         nd_debug_printf("%s: discard: invalid IP protocol version: %hhx\n",
             tag.c_str(), pkt_data[l2_len]);
@@ -888,8 +899,6 @@ void ndDetectionThread::ProcessPacket(void)
         return;
     }
 
-    l4 = reinterpret_cast<const uint8_t *>(l3 + l3_len);
-
     switch (flow.ip_protocol) {
     case IPPROTO_TCP:
         if (l4_len >= 20) {
@@ -1082,8 +1091,8 @@ void ndDetectionThread::ProcessPacket(void)
         new_flow->ndpi_flow,
         (new_flow->ip_version == 4) ?
             (const uint8_t *)hdr_ip : (const uint8_t *)hdr_ip6,
-        pkt_header->len - l2_len,
-        pkt_header->len,
+        pkt_header->caplen - l2_len,
+        ts_pkt,
         id_src,
         id_dst
     );
