@@ -144,7 +144,7 @@ ndNetlink::ndNetlink(const nd_ifaces &ifaces)
         throw ndNetlinkException(strerror(rc));
     }
 
-    if (bind(nd,
+    if (::bind(nd,
         (struct sockaddr *)&sa, sizeof(struct sockaddr_nl)) < 0) {
         rc = errno;
         nd_printf("Error binding netlink socket: %s\n", strerror(rc));
